/***************************************************************************
 *   Copyright (C) 2003 by Glen Canaday                                    *
 *   gurensan@tampabay.rr.com                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include <qvariant.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qtable.h>
#include <qlayout.h>
#include <qaction.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qstring.h>
#include <qfontmetrics.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qfiledialog.h>
#include <qmessagebox.h>

#include "fretscale.h"

FretScale::FretScale( QWidget* parent, const char* name, WFlags fl )
    : QMainWindow( parent, name, fl )
{
    NUM_FRETS_MAX = 48;
    (void)statusBar();
    printer = new QPrinter();
    
    if ( !name )
	setName( "FretScale" );
    setFocusPolicy( QMainWindow::TabFocus );
    setCentralWidget( new QWidget( this, "qt_central_widget" ) );

    frmDecimals = new QButtonGroup( "# Decimals", centralWidget(), "frmDecimals" );
    frmDecimals->setGeometry( QRect( 10, 150, 101, 71 ) );
    
    rdo3 = new QRadioButton( frmDecimals, "rdo3" );
    rdo3->setGeometry( QRect( 10, 15, 71, 21 ) );
    rdo3->setText("3 (.001)");

    rdo4 = new QRadioButton( frmDecimals, "rdo4" );
    rdo4->setGeometry( QRect( 10, 40, 71, 21 ) );
    rdo4->setChecked( TRUE );
    rdo4->setText("4 (.0001)");
    
    lblFrets = new QLabel( centralWidget(), "lblFrets" );
    lblFrets->setGeometry( QRect( 10, 90, 101, 20 ) );
    lblFrets->setText ( "# Frets" );
    
    lblScale = new QLabel( centralWidget(), "lblScale" );
    lblScale->setGeometry( QRect( 10, 20, 101, 21 ) );
    lblScale->setText( "Scale Length" );
    
    btnCalculate = new QPushButton( centralWidget(), "btnCalculate" );
    btnCalculate->setGeometry( QRect( 10, 240, 110, 31 ) );
    btnCalculate->setText( "&Calculate..." );
    
    leFrets = new QLineEdit( centralWidget(), "leFrets" );
    leFrets->setGeometry( QRect( 10, 110, 100, 21 ) );

    leScale = new QLineEdit( centralWidget(), "leScale" );
    leScale->setGeometry( QRect( 10, 40, 101, 21 ) );

    tblResults = new QTable( centralWidget(), "tblResults" );
    tblResults->setNumCols( tblResults->numCols() + 1 );
    tblResults->horizontalHeader()->setLabel( tblResults->numCols() - 1, tr( "Fret Distance" ) );
    tblResults->setNumCols( tblResults->numCols() + 1 );
    tblResults->horizontalHeader()->setLabel( tblResults->numCols() - 1, tr( "Distance from Nut" ) );
    tblResults->setNumCols( tblResults->numCols() + 1 );
    tblResults->horizontalHeader()->setLabel( tblResults->numCols() - 1, tr( "Remaining Scale" ) );
    tblResults->setGeometry( QRect( 130, 10, 430, 360 ) );
    tblResults->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, 0, 0, tblResults->sizePolicy().hasHeightForWidth() ) );
    tblResults->setLineWidth( 2 );
    tblResults->setMargin( 0 );
    tblResults->setResizePolicy( QTable::Default );
    tblResults->setNumRows( NUM_FRETS_MAX );
    tblResults->setNumCols( 3 );
    tblResults->setColumnMovingEnabled( FALSE );
    tblResults->setReadOnly( TRUE );
    tblResults->setColumnWidth(0, QFontMetrics(tblResults->font()).width( "Fret Distance    " ));
    tblResults->setColumnWidth(1, QFontMetrics(tblResults->font()).width( "Distance from Nut    " ));
    tblResults->setColumnWidth(2, QFontMetrics(tblResults->font()).width( "Remaining Scale    " ));
    
    // actions
    fileSaveAsAction = new QAction( this, "fileSaveAsAction" );
    fileSaveAsAction->setMenuText( "Save &As..." );
    filePrintAction = new QAction( this, "filePrintAction" );
    filePrintAction->setMenuText( "&Print..." );
    fileExitAction = new QAction( this, "fileExitAction" );
    fileExitAction->setMenuText( "&Quit..." );
    
    // menubar
    MenuBar = new QMenuBar( this, "MenuBar" );

    MenuBar->setGeometry( QRect( 0, 0, 563, 23 ) );

    fileMenu = new QPopupMenu( this );
    fileSaveAsAction->addTo( fileMenu );
    fileMenu->insertSeparator();
    filePrintAction->addTo( fileMenu );
    fileMenu->insertSeparator();
    fileExitAction->addTo( fileMenu );
    MenuBar->insertItem( QString("&Action"), fileMenu, 2 );

    resize( QSize(563, 419).expandedTo(minimumSizeHint()) );
    
    clearWState( WState_Polished );

    // signals and slots connections
    connect( fileSaveAsAction, SIGNAL( activated() ), this, SLOT( fileSaveAs() ) );
    connect( filePrintAction, SIGNAL( activated() ), this, SLOT( filePrint() ) );
    connect( fileExitAction, SIGNAL( activated() ), this, SLOT( fileExit() ) );
    connect( btnCalculate, SIGNAL( clicked() ), this, SLOT( calculate() ) );

    // tab order
    setTabOrder( leScale, leFrets );
    setTabOrder( leFrets, rdo3);
    setTabOrder( rdo3, rdo4);
    setTabOrder( rdo4, btnCalculate );
    setTabOrder( btnCalculate, leScale);    
    
    numFrets = 0;
}

FretScale::~FretScale(){}

void FretScale::calculate()
{
    int i;
    if (rdo4->isOn()){
	decimals = 4;
    }else{
	decimals = 3;
    }
    if (numFrets > 0 ){
        for (i = 0;i < numFrets;i++){
            tblResults->setText(i, 0, "");
            tblResults->setText(i, 1, "");
            tblResults->setText(i, 2, "");
        }    
    }
    numFrets = QString(leFrets->text()).toInt();    
    if (numFrets < 0) numFrets *= -1;    
    if (numFrets > NUM_FRETS_MAX) numFrets = NUM_FRETS_MAX;    
    if (numFrets == 0) return;

    scale = QString(leScale->text()).toFloat();
    if (scale < 0) scale *= -1;    
    if (scale == 0) return;
    
    remaining = scale;
    
    for (i = 0;i < numFrets;i++){
	fret = remaining / 17.817;
	remaining = remaining - fret;
	if (decimals == 4){
	    tblResults->setText(i, 0, QString().setNum(fret, 'f', 4));
	    tblResults->setText(i, 1, QString().setNum(scale - remaining, 'f', 4));
	    tblResults->setText(i, 2, QString().setNum(remaining, 'f', 4));
	}else{
	    tblResults->setText(i, 0, QString().setNum(fret, 'f', 3));
	    tblResults->setText(i, 1, QString().setNum(scale - remaining, 'f', 3));
	    tblResults->setText(i, 2, QString().setNum(remaining, 'f', 3));
	}
    }
}

void FretScale::fileSaveAs(){
	int i;
	QString filename;
	QString fn = QFileDialog::getSaveFileName( QString::null, QString::null,
					       this );
	if (numFrets == 0 ) return;
	if ( !fn.isEmpty() ) {
		filename = fn;
		QFile f( filename );
		if ( !f.open( IO_WriteOnly ) ) {
			QMessageBox::warning( this, "Fretscale ver. 2.0", QString("Could not write to " + filename), 1, 0, 0);
			return;
		}
	
		QTextStream t( &f );
		t << "Fret  Fret Distance    From Nut    Remaining Scale\n";
		if ( decimals == 4){
			for (i = 0;i < numFrets;i++){
				t << QString().setNum(i + 1);
				t << "\t";
				t << tblResults->text(i, 0);
				t << "\t\t";
				t << tblResults->text(i, 1);
				t << "\t\t";
				t << tblResults->text(i, 2);
				t << "\n";
			}
		}else{
			for (i = 0;i < numFrets;i++){
				t << QString().setNum(i + 1);
				t << "\t";
				t << tblResults->text(i, 0);
				t << "\t\t";
				t << tblResults->text(i, 1);
				t << "\t\t";
				t << tblResults->text(i, 2);
				t << "\n";
			}
		}
		f.close();
		setCaption( filename );
	}
}

void FretScale::filePrint(){
    const int Margin = 10;
    int i;
    QString print_text; // if this is 0, you have nothing to print.
    if (numFrets == 0 ) return;
    if ( printer->setup(this) ) {		// printer dialog
	QPainter p;
	if( !p.begin( printer ) )               // paint on printer
	    return;

	p.setFont( tblResults->font() );
	int yPos	= 0;			// y-position for each line
	QFontMetrics fm = p.fontMetrics();
	QPaintDeviceMetrics metrics( printer ); // need width/height
						// of printer surface
	p.drawText( Margin, Margin + yPos,metrics.width(), 
                       fm.lineSpacing(),
                       ExpandTabs | DontClip,
                       "Fret Number     Fret Distance    Distance from Nut    Remaining Scale",
                       yPos = yPos + fm.lineSpacing());	
	
	for( i = 0; i < numFrets; i++ ) {
	    if ( Margin + yPos > metrics.height() - Margin ) {
		printer->newPage();		// no more room on this page
		yPos = 0;			// back to top of page
	    }
	    print_text = QString().setNum(i) + "\t" + tblResults->text(i, 0) + "\t" + tblResults->text(i, 1) + "\t" + tblResults->text(i, 2) + "\n";
	    p.drawText( Margin, Margin + yPos,metrics.width(), 
                       fm.lineSpacing(),
                       ExpandTabs | DontClip,
                       print_text,
                       yPos = yPos + fm.lineSpacing());
	}
	p.end();				// send job to printer
    }
}

void FretScale::fileExit(){close();}
